/* ***********************************************************
 * This file was automatically generated on 2021-05-06.      *
 *                                                           *
 * Java Bindings Version 2.1.31                              *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package com.tinkerforge;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 3.3cm (1.3") OLED display with 128x64 pixels
 */
public class BrickletOLED128x64 extends Device {
	public final static int DEVICE_IDENTIFIER = 263;
	public final static String DEVICE_DISPLAY_NAME = "OLED 128x64 Bricklet";

	public final static byte FUNCTION_WRITE = (byte)1;
	public final static byte FUNCTION_NEW_WINDOW = (byte)2;
	public final static byte FUNCTION_CLEAR_DISPLAY = (byte)3;
	public final static byte FUNCTION_SET_DISPLAY_CONFIGURATION = (byte)4;
	public final static byte FUNCTION_GET_DISPLAY_CONFIGURATION = (byte)5;
	public final static byte FUNCTION_WRITE_LINE = (byte)6;
	public final static byte FUNCTION_GET_IDENTITY = (byte)255;



	public class DisplayConfiguration {
		public short contrast;
		public boolean invert;

		public String toString() {
			return "[" + "contrast = " + contrast + ", " + "invert = " + invert + "]";
		}
	}

	/**
	 * Creates an object with the unique device ID \c uid. and adds it to
	 * the IP Connection \c ipcon.
	 */
	public BrickletOLED128x64(String uid, IPConnection ipcon) {
		super(uid, ipcon);

		apiVersion[0] = 2;
		apiVersion[1] = 0;
		apiVersion[2] = 0;

		deviceIdentifier = DEVICE_IDENTIFIER;
		deviceDisplayName = DEVICE_DISPLAY_NAME;

		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_WRITE)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_NEW_WINDOW)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_CLEAR_DISPLAY)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_SET_DISPLAY_CONFIGURATION)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_GET_DISPLAY_CONFIGURATION)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_WRITE_LINE)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_GET_IDENTITY)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;

		ipcon.addDevice(this);
	}

	/**
	 * Appends 64 byte of data to the window as set by {@link BrickletOLED128x64#newWindow(short, short, short, short)}.
	 * 
	 * Each row has a height of 8 pixels which corresponds to one byte of data.
	 * 
	 * Example: if you call {@link BrickletOLED128x64#newWindow(short, short, short, short)} with column from 0 to 127 and row
	 * from 0 to 7 (the whole display) each call of {@link BrickletOLED128x64#write(short[])} (red arrow) will
	 * write half of a row.
	 * 
	 * .. image:: /Images/Bricklets/bricklet_oled_128x64_display.png
	 *    :scale: 100 %
	 *    :alt: Display pixel order
	 *    :align: center
	 *    :target: ../../_images/Bricklets/bricklet_oled_128x64_display.png
	 * 
	 * The LSB (D0) of each data byte is at the top and the MSB (D7) is at the
	 * bottom of the row.
	 * 
	 * The next call of {@link BrickletOLED128x64#write(short[])} will write the second half of the row
	 * and the next two the second row and so on. To fill the whole display
	 * you need to call {@link BrickletOLED128x64#write(short[])} 16 times.
	 */
	public void write(short[] data) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)72, FUNCTION_WRITE, this);

		for (int i = 0; i < 64; i++) {
			bb.put((byte)data[i]);
		}

		sendRequest(bb.array(), 0);
	}

	/**
	 * Sets the window in which you can write with {@link BrickletOLED128x64#write(short[])}. One row
	 * has a height of 8 pixels.
	 */
	public void newWindow(short columnFrom, short columnTo, short rowFrom, short rowTo) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)12, FUNCTION_NEW_WINDOW, this);

		bb.put((byte)columnFrom);
		bb.put((byte)columnTo);
		bb.put((byte)rowFrom);
		bb.put((byte)rowTo);

		sendRequest(bb.array(), 0);
	}

	/**
	 * Clears the current content of the window as set by {@link BrickletOLED128x64#newWindow(short, short, short, short)}.
	 */
	public void clearDisplay() throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_CLEAR_DISPLAY, this);


		sendRequest(bb.array(), 0);
	}

	/**
	 * Sets the configuration of the display.
	 * 
	 * You can set a contrast value from 0 to 255 and you can invert the color
	 * (black/white) of the display.
	 */
	public void setDisplayConfiguration(short contrast, boolean invert) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)10, FUNCTION_SET_DISPLAY_CONFIGURATION, this);

		bb.put((byte)contrast);
		bb.put((byte)(invert ? 1 : 0));

		sendRequest(bb.array(), 0);
	}

	/**
	 * Returns the configuration as set by {@link BrickletOLED128x64#setDisplayConfiguration(short, boolean)}.
	 */
	public DisplayConfiguration getDisplayConfiguration() throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_DISPLAY_CONFIGURATION, this);


		byte[] response = sendRequest(bb.array(), 10);

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		DisplayConfiguration obj = new DisplayConfiguration();
		obj.contrast = IPConnectionBase.unsignedByte(bb.get());
		obj.invert = (bb.get()) != 0;

		return obj;
	}

	/**
	 * Writes text to a specific line with a specific position.
	 * The text can have a maximum of 26 characters.
	 * 
	 * For example: (1, 10, "Hello") will write *Hello* in the middle of the
	 * second line of the display.
	 * 
	 * You can draw to the display with {@link BrickletOLED128x64#write(short[])} and then add text to it
	 * afterwards.
	 * 
	 * The display uses a special 5x7 pixel charset. You can view the characters
	 * of the charset in Brick Viewer.
	 */
	public void writeLine(short line, short position, String text) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)36, FUNCTION_WRITE_LINE, this);

		bb.put((byte)line);
		bb.put((byte)position);
		for (int i = 0; i < 26; i++) {
			try {
				bb.put((byte)text.charAt(i));
			} catch(Exception e) {
				bb.put((byte)0);
			}
		}

		sendRequest(bb.array(), 0);
	}

	/**
	 * Returns the UID, the UID where the Bricklet is connected to,
	 * the position, the hardware and firmware version as well as the
	 * device identifier.
	 * 
	 * The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
	 * A Bricklet connected to an :ref:`Isolator Bricklet &lt;isolator_bricklet&gt;` is always at
	 * position 'z'.
	 * 
	 * The device identifier numbers can be found :ref:`here &lt;device_identifier&gt;`.
	 * |device_identifier_constant|
	 */
	public Identity getIdentity() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_IDENTITY, this);


		byte[] response = sendRequest(bb.array(), 33);

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		Identity obj = new Identity();
		obj.uid = IPConnectionBase.string(bb, 8);
		obj.connectedUid = IPConnectionBase.string(bb, 8);
		obj.position = (char)(bb.get());
		for (int i = 0; i < 3; i++) {
			obj.hardwareVersion[i] = IPConnectionBase.unsignedByte(bb.get());
		}
		for (int i = 0; i < 3; i++) {
			obj.firmwareVersion[i] = IPConnectionBase.unsignedByte(bb.get());
		}
		obj.deviceIdentifier = IPConnectionBase.unsignedShort(bb.getShort());

		return obj;
	}
}
