/* ***********************************************************
 * This file was automatically generated on 2021-05-06.      *
 *                                                           *
 * Java Bindings Version 2.1.31                              *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package com.tinkerforge;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Controls remote mains switches
 */
public class BrickletRemoteSwitch extends Device {
	public final static int DEVICE_IDENTIFIER = 235;
	public final static String DEVICE_DISPLAY_NAME = "Remote Switch Bricklet";

	public final static byte FUNCTION_SWITCH_SOCKET = (byte)1;
	public final static byte FUNCTION_GET_SWITCHING_STATE = (byte)2;
	public final static byte FUNCTION_SET_REPEATS = (byte)4;
	public final static byte FUNCTION_GET_REPEATS = (byte)5;
	public final static byte FUNCTION_SWITCH_SOCKET_A = (byte)6;
	public final static byte FUNCTION_SWITCH_SOCKET_B = (byte)7;
	public final static byte FUNCTION_DIM_SOCKET_B = (byte)8;
	public final static byte FUNCTION_SWITCH_SOCKET_C = (byte)9;
	public final static byte FUNCTION_GET_IDENTITY = (byte)255;
	private final static int CALLBACK_SWITCHING_DONE = 3;

	public final static short SWITCH_TO_OFF = (short)0;
	public final static short SWITCH_TO_ON = (short)1;
	public final static short SWITCHING_STATE_READY = (short)0;
	public final static short SWITCHING_STATE_BUSY = (short)1;

	private List<SwitchingDoneListener> listenerSwitchingDone = new CopyOnWriteArrayList<SwitchingDoneListener>();

	/**
	 * This listener is triggered whenever the switching state changes
	 * from busy to ready, see {@link BrickletRemoteSwitch#getSwitchingState()}.
	 */
	public interface SwitchingDoneListener extends DeviceListener {
		public void switchingDone();
	}

	/**
	 * Creates an object with the unique device ID \c uid. and adds it to
	 * the IP Connection \c ipcon.
	 */
	public BrickletRemoteSwitch(String uid, IPConnection ipcon) {
		super(uid, ipcon);

		apiVersion[0] = 2;
		apiVersion[1] = 0;
		apiVersion[2] = 1;

		deviceIdentifier = DEVICE_IDENTIFIER;
		deviceDisplayName = DEVICE_DISPLAY_NAME;

		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_SWITCH_SOCKET)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_GET_SWITCHING_STATE)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_SET_REPEATS)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_GET_REPEATS)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_SWITCH_SOCKET_A)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_SWITCH_SOCKET_B)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_DIM_SOCKET_B)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_SWITCH_SOCKET_C)] = RESPONSE_EXPECTED_FLAG_FALSE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_GET_IDENTITY)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;

		callbacks[CALLBACK_SWITCHING_DONE] = new IPConnection.DeviceCallbackListener() {
			public void callback(byte[] packet) {
				if (packet.length != 8) {
					return; // silently ignoring callback with wrong length
				}

				for (SwitchingDoneListener listener: listenerSwitchingDone) {
					listener.switchingDone();
				}
			}
		};

		ipcon.addDevice(this);
	}

	/**
	 * This function is deprecated, use {@link BrickletRemoteSwitch#switchSocketA(short, short, short)} instead.
	 */
	public void switchSocket(short houseCode, short receiverCode, short switchTo) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)11, FUNCTION_SWITCH_SOCKET, this);

		bb.put((byte)houseCode);
		bb.put((byte)receiverCode);
		bb.put((byte)switchTo);

		sendRequest(bb.array(), 0);
	}

	/**
	 * Returns the current switching state. If the current state is busy, the
	 * Bricklet is currently sending a code to switch a socket. It will not
	 * accept any requests to switch sockets until the state changes to ready.
	 * 
	 * How long the switching takes is dependent on the number of repeats, see
	 * {@link BrickletRemoteSwitch#setRepeats(short)}.
	 */
	public short getSwitchingState() throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_SWITCHING_STATE, this);


		byte[] response = sendRequest(bb.array(), 9);

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short state = IPConnectionBase.unsignedByte(bb.get());

		return state;
	}

	/**
	 * Sets the number of times the code is sent when one of the switch socket
	 * functions is called. The repeats basically correspond to the amount of time
	 * that a button of the remote is pressed.
	 * 
	 * Some dimmers are controlled by the length of a button pressed,
	 * this can be simulated by increasing the repeats.
	 */
	public void setRepeats(short repeats) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)9, FUNCTION_SET_REPEATS, this);

		bb.put((byte)repeats);

		sendRequest(bb.array(), 0);
	}

	/**
	 * Returns the number of repeats as set by {@link BrickletRemoteSwitch#setRepeats(short)}.
	 */
	public short getRepeats() throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_REPEATS, this);


		byte[] response = sendRequest(bb.array(), 9);

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short repeats = IPConnectionBase.unsignedByte(bb.get());

		return repeats;
	}

	/**
	 * To switch a type A socket you have to give the house code, receiver code and the
	 * state (on or off) you want to switch to.
	 * 
	 * A detailed description on how you can figure out the house and receiver code
	 * can be found :ref:`here &lt;remote_switch_bricklet_type_a_house_and_receiver_code&gt;`.
	 * 
	 * .. versionadded:: 2.0.1$nbsp;(Plugin)
	 */
	public void switchSocketA(short houseCode, short receiverCode, short switchTo) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)11, FUNCTION_SWITCH_SOCKET_A, this);

		bb.put((byte)houseCode);
		bb.put((byte)receiverCode);
		bb.put((byte)switchTo);

		sendRequest(bb.array(), 0);
	}

	/**
	 * To switch a type B socket you have to give the address, unit and the state
	 * (on or off) you want to switch to.
	 * 
	 * To switch all devices with the same address use 255 for the unit.
	 * 
	 * A detailed description on how you can teach a socket the address and unit can
	 * be found :ref:`here &lt;remote_switch_bricklet_type_b_address_and_unit&gt;`.
	 * 
	 * .. versionadded:: 2.0.1$nbsp;(Plugin)
	 */
	public void switchSocketB(long address, short unit, short switchTo) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)14, FUNCTION_SWITCH_SOCKET_B, this);

		bb.putInt((int)address);
		bb.put((byte)unit);
		bb.put((byte)switchTo);

		sendRequest(bb.array(), 0);
	}

	/**
	 * To control a type B dimmer you have to give the address, unit and the
	 * dim value you want to set the dimmer to.
	 * 
	 * A detailed description on how you can teach a dimmer the address and unit can
	 * be found :ref:`here &lt;remote_switch_bricklet_type_b_address_and_unit&gt;`.
	 * 
	 * .. versionadded:: 2.0.1$nbsp;(Plugin)
	 */
	public void dimSocketB(long address, short unit, short dimValue) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)14, FUNCTION_DIM_SOCKET_B, this);

		bb.putInt((int)address);
		bb.put((byte)unit);
		bb.put((byte)dimValue);

		sendRequest(bb.array(), 0);
	}

	/**
	 * To switch a type C socket you have to give the system code, device code and the
	 * state (on or off) you want to switch to.
	 * 
	 * A detailed description on how you can figure out the system and device code
	 * can be found :ref:`here &lt;remote_switch_bricklet_type_c_system_and_device_code&gt;`.
	 * 
	 * .. versionadded:: 2.0.1$nbsp;(Plugin)
	 */
	public void switchSocketC(char systemCode, short deviceCode, short switchTo) throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)11, FUNCTION_SWITCH_SOCKET_C, this);

		bb.put((byte)systemCode);
		bb.put((byte)deviceCode);
		bb.put((byte)switchTo);

		sendRequest(bb.array(), 0);
	}

	/**
	 * Returns the UID, the UID where the Bricklet is connected to,
	 * the position, the hardware and firmware version as well as the
	 * device identifier.
	 * 
	 * The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
	 * A Bricklet connected to an :ref:`Isolator Bricklet &lt;isolator_bricklet&gt;` is always at
	 * position 'z'.
	 * 
	 * The device identifier numbers can be found :ref:`here &lt;device_identifier&gt;`.
	 * |device_identifier_constant|
	 */
	public Identity getIdentity() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_IDENTITY, this);


		byte[] response = sendRequest(bb.array(), 33);

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		Identity obj = new Identity();
		obj.uid = IPConnectionBase.string(bb, 8);
		obj.connectedUid = IPConnectionBase.string(bb, 8);
		obj.position = (char)(bb.get());
		for (int i = 0; i < 3; i++) {
			obj.hardwareVersion[i] = IPConnectionBase.unsignedByte(bb.get());
		}
		for (int i = 0; i < 3; i++) {
			obj.firmwareVersion[i] = IPConnectionBase.unsignedByte(bb.get());
		}
		obj.deviceIdentifier = IPConnectionBase.unsignedShort(bb.getShort());

		return obj;
	}

	/**
	 * Adds a SwitchingDone listener.
	 */
	public void addSwitchingDoneListener(SwitchingDoneListener listener) {
		listenerSwitchingDone.add(listener);
	}

	/**
	 * Removes a SwitchingDone listener.
	 */
	public void removeSwitchingDoneListener(SwitchingDoneListener listener) {
		listenerSwitchingDone.remove(listener);
	}
}
