/* ***********************************************************
 * This file was automatically generated on 2021-05-06.      *
 *                                                           *
 * Java Bindings Version 2.1.31                              *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

package com.tinkerforge;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Detects inclination of Bricklet (tilt switch open/closed)
 */
public class BrickletTilt extends Device {
	public final static int DEVICE_IDENTIFIER = 239;
	public final static String DEVICE_DISPLAY_NAME = "Tilt Bricklet";

	public final static byte FUNCTION_GET_TILT_STATE = (byte)1;
	public final static byte FUNCTION_ENABLE_TILT_STATE_CALLBACK = (byte)2;
	public final static byte FUNCTION_DISABLE_TILT_STATE_CALLBACK = (byte)3;
	public final static byte FUNCTION_IS_TILT_STATE_CALLBACK_ENABLED = (byte)4;
	public final static byte FUNCTION_GET_IDENTITY = (byte)255;
	private final static int CALLBACK_TILT_STATE = 5;

	public final static short TILT_STATE_CLOSED = (short)0;
	public final static short TILT_STATE_OPEN = (short)1;
	public final static short TILT_STATE_CLOSED_VIBRATING = (short)2;

	private List<TiltStateListener> listenerTiltState = new CopyOnWriteArrayList<TiltStateListener>();

	/**
	 * This listener provides the current tilt state. It is called every time the
	 * state changes.
	 * 
	 * See {@link BrickletTilt#getTiltState()} for a description of the states.
	 */
	public interface TiltStateListener extends DeviceListener {
		public void tiltState(short state);
	}

	/**
	 * Creates an object with the unique device ID \c uid. and adds it to
	 * the IP Connection \c ipcon.
	 */
	public BrickletTilt(String uid, IPConnection ipcon) {
		super(uid, ipcon);

		apiVersion[0] = 2;
		apiVersion[1] = 0;
		apiVersion[2] = 0;

		deviceIdentifier = DEVICE_IDENTIFIER;
		deviceDisplayName = DEVICE_DISPLAY_NAME;

		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_GET_TILT_STATE)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_ENABLE_TILT_STATE_CALLBACK)] = RESPONSE_EXPECTED_FLAG_TRUE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_DISABLE_TILT_STATE_CALLBACK)] = RESPONSE_EXPECTED_FLAG_TRUE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_IS_TILT_STATE_CALLBACK_ENABLED)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;
		responseExpected[IPConnectionBase.unsignedByte(FUNCTION_GET_IDENTITY)] = RESPONSE_EXPECTED_FLAG_ALWAYS_TRUE;

		callbacks[CALLBACK_TILT_STATE] = new IPConnection.DeviceCallbackListener() {
			public void callback(byte[] packet) {
				if (packet.length != 9) {
					return; // silently ignoring callback with wrong length
				}

				ByteBuffer bb = ByteBuffer.wrap(packet, 8, packet.length - 8);
				bb.order(ByteOrder.LITTLE_ENDIAN);

				short state = IPConnectionBase.unsignedByte(bb.get());

				for (TiltStateListener listener: listenerTiltState) {
					listener.tiltState(state);
				}
			}
		};

		ipcon.addDevice(this);
	}

	/**
	 * Returns the current tilt state. The state can either be
	 * 
	 * * 0 = Closed: The ball in the tilt switch closes the circuit.
	 * * 1 = Open: The ball in the tilt switch does not close the circuit.
	 * * 2 = Closed Vibrating: The tilt switch is in motion (rapid change between open and close).
	 * 
	 * .. image:: /Images/Bricklets/bricklet_tilt_mechanics.jpg
	 *    :scale: 100 %
	 *    :alt: Tilt states
	 *    :align: center
	 *    :target: ../../_images/Bricklets/bricklet_tilt_mechanics.jpg
	 */
	public short getTiltState() throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_TILT_STATE, this);


		byte[] response = sendRequest(bb.array(), 9);

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short state = IPConnectionBase.unsignedByte(bb.get());

		return state;
	}

	/**
	 * Enables the {@link BrickletTilt.TiltStateListener} listener.
	 */
	public void enableTiltStateCallback() throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_ENABLE_TILT_STATE_CALLBACK, this);


		sendRequest(bb.array(), 0);
	}

	/**
	 * Disables the {@link BrickletTilt.TiltStateListener} listener.
	 */
	public void disableTiltStateCallback() throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_DISABLE_TILT_STATE_CALLBACK, this);


		sendRequest(bb.array(), 0);
	}

	/**
	 * Returns *true* if the {@link BrickletTilt.TiltStateListener} listener is enabled.
	 */
	public boolean isTiltStateCallbackEnabled() throws TinkerforgeException {
		checkValidity();

		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_IS_TILT_STATE_CALLBACK_ENABLED, this);


		byte[] response = sendRequest(bb.array(), 9);

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		boolean enabled = (bb.get()) != 0;

		return enabled;
	}

	/**
	 * Returns the UID, the UID where the Bricklet is connected to,
	 * the position, the hardware and firmware version as well as the
	 * device identifier.
	 * 
	 * The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
	 * A Bricklet connected to an :ref:`Isolator Bricklet &lt;isolator_bricklet&gt;` is always at
	 * position 'z'.
	 * 
	 * The device identifier numbers can be found :ref:`here &lt;device_identifier&gt;`.
	 * |device_identifier_constant|
	 */
	public Identity getIdentity() throws TinkerforgeException {
		ByteBuffer bb = ipcon.createRequestPacket((byte)8, FUNCTION_GET_IDENTITY, this);


		byte[] response = sendRequest(bb.array(), 33);

		bb = ByteBuffer.wrap(response, 8, response.length - 8);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		Identity obj = new Identity();
		obj.uid = IPConnectionBase.string(bb, 8);
		obj.connectedUid = IPConnectionBase.string(bb, 8);
		obj.position = (char)(bb.get());
		for (int i = 0; i < 3; i++) {
			obj.hardwareVersion[i] = IPConnectionBase.unsignedByte(bb.get());
		}
		for (int i = 0; i < 3; i++) {
			obj.firmwareVersion[i] = IPConnectionBase.unsignedByte(bb.get());
		}
		obj.deviceIdentifier = IPConnectionBase.unsignedShort(bb.getShort());

		return obj;
	}

	/**
	 * Adds a TiltState listener.
	 */
	public void addTiltStateListener(TiltStateListener listener) {
		listenerTiltState.add(listener);
	}

	/**
	 * Removes a TiltState listener.
	 */
	public void removeTiltStateListener(TiltStateListener listener) {
		listenerTiltState.remove(listener);
	}
}
